!     ------------------------------------------------------------------
!
!     main.f95
!
!
!     Created on Sun Apr 08 16:07:51 2007
!     Copyright (c) 2007 James K Beard. All rights reserved.
!
!
!     ------------------------------------------------------------------

program main	!Generate v-i curve family for BJT
use BJT, only:Saturation_Resistance,V_CE_Early,beta,IC0,	&
  IBdelta,NIBase,V_CEDelta,V_CEMax,BJT_vi_Curves,BJT_Output_Admittance
implicit none
!Elementary model
!Saturation equation is IC = VC/Saturation_resistance
!Constant current equation is IC = ISat+beta*IB+VC/Collector_Resistance
!The smaller of these is the output
!The intersection of these lines is always included in the output table

character(len=40)::fname="BJT_transistor_vi",input_file="BJT_input_data.txt",rmf="ReadMe2Use.txt"
integer::lunin=10,lunout=11,lunreadme=12,lundataf=13

logical::already_called
!real(kind(0.d0))::VDrain,VDIntercept1,VDInterceptN,VDDelta_start


  inquire(file=trim(input_file),exist=already_called)
  if(.NOT.already_called) then

!Define transistor parameters
    beta=350
    Saturation_Resistance=45d0	!45 Ohms
    V_CE_Early=100d0		!100 Volts
    IC0=200d-9			!200 pA zero-bias collector current
!Define plotting paramters

    IBDelta=10.d-6		!10 uA
    NIBase=6			!0 to 60 uA

    V_CEDelta=1.d0		!1 Volt increments
    V_CEMax=20d0			!0 to 20 Volts

    open(lundataf,file=trim(input_file))
    write(lundataf,1000) beta,Saturation_Resistance,	&
      V_CE_Early,IC0*1d9,IBDelta*1d6,NIBase,V_CEDelta,V_CEMax
    close(lundataf)
    open(lunreadme,file=trim(rmf))
    write(lunreadme,3000) trim(input_file),trim(input_file),trim(fname)
    close(lunreadme)

  else
!Read the transistor and plotting parameters from an input file
    open(lundataf,file=trim(input_file))
    read(lundataf,2000) beta,Saturation_Resistance,	&
      V_CE_Early,IC0,IBDelta,NIBase,V_CEDelta,V_CEMax
    close(lundataf)
    IC0=IC0/1d9		!Convert from pA to Amps
    IBDelta=IBDelta/1d6	!Convert from uA to Amps
  end if

!Plot the curves
  call BJT_vi_Curves(lunout)
  print *,"Output admittance, V_CE=10 V, IC = 1 mA:",	&
    BJT_Output_Admittance(10d0,1d-3/beta)*1d6," micromhos"

  print *,"Normal exit."

1000 format("Data file for BJT model and v-i courve plotter"/	&
  g25.15,",beta"/	&
  g25.15,",Saturation_Resistance, Ohms; typically 100 Ohms or less"/	&
  g25.15,",Early voltage, typically 50 Volts to 100 Volts"/	&
  g25.15,",IC0, picoamps; collector leakage current"/			&
  g25.15,",IBDelta, microamps; difference in base currents for v-i plots"/	&
  I25,",NIBasee, number of v-i plots; IBase = i*IBDelta+VTransistion, i=0,NIBase"/	&
  g25.15,",V_CEDelta, Volts; plot granularity in constant current region"/	&
  g25.15,",V_CEMax, Volts; maximum drain voltage for v-i plots"/)
2000 format(/5(g25.15/),i25/2(g25.15/))
3000 format("Program copyright 2007 by James K Beard."/				&
  "There are no warranties on this program, express or implied."//		&
  "To use, edit the file ",a//							&
  "If the program is called and ",a," is not present, it generates a new"/	&
  "one with default parameters."//						&
  "The first four parameters determine the transistor characteristics."/	&
  "The next two parameters determine the range of base currents used."/		&
  "The last two parameters determine the maximum collector voltage plotted"/	&
  "and the granularity of the data in the constant current region.  The"/	&
  "program automatically adjusts the granularity for the saturation region."//	&
  "If the Early voltage is not given but the output admittance is provided"/	&
  "collector current, the Early voltage is the collector current divided"/	&
  "by the output admittance."//							&
  "To plot the v-i curves, load the output file ",a,".csv"/			&
  "into a spreadsheet program.  Click on the Chart Wizard, select chart"/	&
  "type ""XY (scatter)"", of type ""data points connected by lines"/		&
  "without markers."""/)

end program main
